package sos.net;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Vector;

import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTP;


/**
 * <p>Title: FTP-Client</p>
 * <p>Description: a wrapper around the Jakarta Commons FTP</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: SOS GmbH</p>
 * @author <a href="mailto:ghassan.beydoun@sos-berlin.com">Ghassan Beydoun</a>
 * @version $Id: SOSFTP.java 2879 2007-08-06 11:19:08Z mo $
 * @resource commons-net-1.3.0.jar
 */


public class SOSFTP extends FTPClient {

	private Vector fileListNames = new Vector();

	public SOSFTP() {}

	/**
	 * SOSFTP constructor: there is no need to call the connect-Method
	 * @param host the remote ftp server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 */
	public SOSFTP ( String host ) throws SocketException,IOException,UnknownHostException {
		connect( host);
	}

	/**
	 * SOSFTP constructor: there is no need to call the connect-Method
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 */
	public SOSFTP ( String host, int port ) throws SocketException,IOException,UnknownHostException {
		connect( host, port);
	}

	/*
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
 	 * @see org.apache.commons.net.SocketClient#connect(java.lang.String, int)
	 */
	public void connect(String host, int port) throws SocketException,IOException,UnknownHostException {
		if ( !isConnected())
		super.connect(host, port);
	}

	/*
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 * @see org.apache.commons.net.SocketClient#connect(java.lang.String)
	 */
	public void connect(String arg0) throws SocketException, IOException,UnknownHostException {
		if ( !isConnected())
		super.connect(arg0);
        
        
	}


	  /**
	   * Creates a new subdirectory on the FTP server in the current directory .
	   * @param pathname The pathname of the directory to create.
	   * @return True if successfully completed, false if not.
	   * @throws java.lang.IOException
	   */
	  public boolean mkdir( String pathname ) throws IOException {
	    return makeDirectory(pathname);
	  }

	  /**
	   * Removes a directory on the FTP server (if empty).
	   * @param pathname The pathname of the directory to remove.
	   * @return True if successfully completed, false if not.
	   * @throws java.lang.IOException
	   */
	  public boolean rmdir( String pathname ) throws IOException {
	    return removeDirectory( pathname );
	  }

	  /**
	  * turn passive transfer mode on.
	  *
	  * @return The reply code received from the server.
	  */
	 public int passive() throws IOException {
       return pasv();
	 }


	  /**
	   * return a listing of the contents of a directory in short format on
	   * the remote machine
	   * @param pathname on remote machine
	   * @return a listing of the contents of a directory on the remote machine
	   *
	   * @exception Exception
	   * @see #dir()
	   */
	  public Vector nList(String pathname) throws Exception {
	  	String[] fileList = null;
  	    Vector fileListNames = new Vector();
	      fileList = listNames(pathname);	      
          if ( fileList == null )
              return fileListNames;
	      for(int i=0;i<fileList.length;i++) {
		  	fileListNames.add( fileList[i] );
		  }
	      return fileListNames;
	    
	  } // nList

	  /**
	   * return a listing of the contents of a directory in short format on
	   * the remote machine
	   * @param pathname on remote machine
	   * @return a listing of the contents of a directory on the remote machine
	   *
	   * @exception Exception
	   * @see #dir()
	   */	  
	  public Vector nList(String pathname, boolean recursive) throws Exception {

		  if(!recursive) {
			  return nList(pathname);
		  } else {
			  
			  FTPFile[] listFiles  = listFiles( pathname );
			  for(int i = 0; i < listFiles.length; i++) {
				  FTPFile file = listFiles[i];
				  if(file.isDirectory()) {
					  nList(pathname + "/" +file.getName(), recursive);
				  }else {
					  fileListNames.add(pathname + "/" +  file.getName() );  
				  }
			  }
			  return fileListNames;

		  }
	  } // nList
	  
	  /**
	   * return a listing of the contents of a directory in short format on
	   * the remote machine
	   *
	   * @return a listing of the contents of a directory on the remote machine
	   *
	   * @exception Exception
	   * @see #nList( String )
	   * @see #dir()
	   * @see #dir( String )
	   */
	  public Vector nList() throws Exception {
		  String[] fileList = null;
		  Vector fileListNames = new Vector();
		  fileList = listNames();
		  if ( fileList == null )
			  return fileListNames;
		  for(int i=0;i<fileList.length;i++) {			  
			  fileListNames.add( fileList[i] );
		  }
		  return fileListNames;
		  
		  
	  } // nList


	  /**
	   * return a listing of the contents of a directory in short format on
	   * the remote machine
	   *
	   * @return a listing of the contents of a directory on the remote machine
	   *
	   * @exception Exception
	   * @see #nList( String )
	   * @see #dir()
	   * @see #dir( String )
	   */
	 
	  public Vector nList(boolean recursive) throws Exception {
		  if(!recursive) {
			  return nList();  
		  } else {
			  FTPFile[] listFiles  = listFiles();
			  for (int i = 0; i < listFiles.length; i++) {
				  FTPFile file = listFiles[i];
				  if(file.isDirectory()) {
					  //this.cd(file.getName());
					  nList(file.getName(), recursive);
				  } else {
					  fileListNames.add( file.getName() );					  
				  }				  
			  }			  
			  return fileListNames;
		  }

	  } // nList
	  /**
	   * return a listing of the files in a directory in long format on
	   * the remote machine
	   * @param pathname on remote machine
	   * @return a listing of the contents of a directory on the remote machine
	   * @exception Exception
	   * @see #nList()
	   * @see #nList( String )
	   * @see #dir()
	   */
	  public Vector dir( String pathname ) throws Exception {

 	    Vector fileList = new Vector();
        try {
	      FTPFile[] listFiles  = listFiles( pathname );
	      for(int i=0;i<listFiles.length;i++) {
            if (listFiles[i].isFile()) {
                fileList.add( listFiles[i].toString() );
            }
		  }
        } catch (Exception e) {
            throw new Exception("error in dir(): " + e.getMessage());
        }
	      return fileList;
	  }


      /**
       * return a listing of a directory in long format on
       * the remote machine
       * @param pathname on remote machine
       * @return a listing of the contents of a directory on the remote machine
       * @exception Exception
       * @see #nList()
       * @see #nList( String )
       * @see #dir()
       */
      public Vector dir( String pathname, int flag ) throws Exception {

        Vector fileList = new Vector();
          FTPFile[] listFiles  = listFiles( pathname );
          for(int i=0;i<listFiles.length;i++) {
              if (flag > 0 && listFiles[i].isDirectory()) {
                 fileList.addAll(this.dir(pathname + "/" + listFiles[i].toString(), ((flag >= 1024) ? flag : flag+1024)));   
              } else {
                  if (flag >= 1024) {
                      fileList.add( pathname + "/" + listFiles[i].toString() );
                  } else {
                      fileList.add( listFiles[i].toString() );
                  }
              }
          }
          return fileList;
      }


	  /**
	   * return a listing of the files of the current directory in long format on
	   * the remote machine
	   * @return a listing of the contents of the current directory on the remote machine
	   * @exception Exception
	   * @see #nList()
	   * @see #nList( String )
	   * @see #dir( String )
	   */
	  public Vector dir() throws Exception {
	  	return dir( "." );
	  }

	  /**
	   * @return The entire text from the last FTP response as a String.
	   */
	  public String getResponse() {
	  	return this.getReplyString();
	  }

	  /**
	   * return the size of remote-file on the remote machine on success, otherwise -1
	   * @param remoteFile the file on remote machine
	   * @return the size of remote-file on remote machine
	   */
	  public long size(String remoteFile ) throws Exception {

	    this.sendCommand("SIZE " + remoteFile);
        if ( this.getReplyCode() == FTPReply.CODE_213 )
	      return Long.parseLong( trimResponseCode( this.getReplyString()) );
	    else
	      return -1L;
	  }

	  /**
	   * trim the response code at the beginning
	   * @param response
	   * @return the response string without response code
	   * @throws Exception
	   */
	  private String trimResponseCode(String response) throws Exception
	    {
	        if (response.length() < 5) return response;
	        return response.substring(4).trim();
	    }

	    /**
	     * Retrieves a named file from the ftp server.
	     *
	     * @param localFile The name of the local file.
	     * @param remoteFile The name of the remote file.
	     * @exception Exception
	     * @see #getFile( String, String )
	     */
	    public boolean get(String remoteFile, String localFile) throws Exception {
	      FileOutputStream out = null;
	      boolean rc = false;
	      try {
	        out = new FileOutputStream( localFile );
	        rc = retrieveFile( remoteFile, out );
	        return rc;
	      }
	      catch (IOException e) {
	        throw e;
	      }
	      finally {
	        try { if ( out != null) out.close(); } catch (Exception e) {}
	      }
	    } // get


	    /**
	     * Retrieves a named file from the ftp server.
	     *
	     * @param localFile The name of the local file.
	     * @param remoteFile The name of the remote file.
	     * @return  The total number of bytes retrieved.
	     * @see #get( String, String )
	     * @exception Exception
	     */
	    public long getFile(String remoteFile, String localFile) throws Exception {
            return getFile(remoteFile, localFile, false);
	    }

        /**
         * Retrieves a named file from the ftp server.
         *
         * @param localFile The name of the local file.
         * @param remoteFile The name of the remote file.
         * @param append Appends the remote file to the local file.
         * @return  The total number of bytes retrieved.
         * @see #get( String, String )
         * @exception Exception
         */
        public long getFile(String remoteFile, String localFile, boolean append) throws Exception {
          InputStream in = null;
          OutputStream out = null;
          long totalBytes = 0;
          try {
            in = retrieveFileStream(remoteFile );
            if ( in == null) return -1L;
            byte[] buffer = new byte[4096];
            out = new FileOutputStream( new File(localFile), append );
            int bytes_read = 0;
            synchronized (this) {
              while ( (bytes_read = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytes_read);
                out.flush();
                totalBytes += bytes_read;
              }
            }
            in.close(); // we must close the stream: connection hang up :-(
            out.close();
            if(!completePendingCommand()) {
                logout();
                disconnect();
                throw (new Exception("File transfer failed."));
            }

            if ( totalBytes > 0 )
                return totalBytes;
            else 
                return -1L;
          } catch( IOException e ) {
            throw e;
          }
          finally {
            try { if ( in != null) in.close(); } catch (Exception e) {}
            try { out.flush(); } catch (Exception e) {}
            try { if ( out != null)  out.close(); } catch (Exception e) {}
          }
        }

	    /**
	     * Stores a file on the server using the given name.
	     * @param localFile The name of the local file.
	     * @param remoteFile The name of the remote file.
	     * @return True if successfully completed, false if not.
	     * @exception Exception
	     * @see #putFile( String, String )
	     */
	    public boolean put(String localFile, String remoteFile) throws Exception {
	      FileInputStream in = null;
	      boolean rc = false;
	      try {
	        in = new FileInputStream( localFile );
	        rc = storeFile( remoteFile, in);
	        return rc;
	      } catch( Exception e ) {
	        throw e;
	      }
	      finally {
	        try { if ( in != null) in.close(); } catch (Exception e) {}
	      }
	    }


	    /**
	     * Stores a file on the server using the given name.
	     * @param localFile The name of the local file.
	     * @param remoteFile The name of the remote file.
	     * @return The total number of bytes written.
	     * @exception Exception
	     * @see #put( String, String )
	     */
	    public long putFile(String localFile, String remoteFile) throws Exception {	   	    	
	    	return putFile(localFile,storeFileStream( remoteFile )); 
	    } // putFile


	    /**
	     *
	     * @param localfile The name of the local file.
	     * @param an OutputStream through which data can be 
	     * written to store a file on the server using the given name.
	     * @return The total number of bytes written.
	     * @exception Exception
	     */
	    public long putFile(String localFile, OutputStream out) throws Exception {
		      FileInputStream in = null;
		      long totalBytes  = 0;
		      try {

		      	if ( out == null)
		      		throw (new Exception("output stream has null value."));

		        byte[] buffer = new byte[4096];
		        in = new FileInputStream( new File(localFile));
		        int bytesWritten;
		        synchronized (this) {
		          while ( (bytesWritten = in.read(buffer)) != -1) {
		          	out.write(buffer, 0, bytesWritten);
		            totalBytes += bytesWritten;
		          }
		        }

		        in.close(); // we must close the stream: connection hang up :-(
		        out.close();
		        if(!completePendingCommand()) {
		            logout();
		            disconnect();
		            throw (new Exception("File transfer failed."));
		        }
		        return totalBytes;
		      } catch( Exception e ) {
		        throw e;
		      }
		      finally {
		        try { if ( in != null) in.close(); } catch (Exception e) {}
		        try { out.flush(); } catch (Exception e) {}
		        try { if ( out != null) out.close(); } catch (Exception e) {}
		      }
		    } // putFile


	    /**
	     * append a local file to the remote one on the server
	     * @param localFile The name of the local file.
	     * @param remoteFile The name of the remote file.
	     * @return The total number of bytes appended.
	     * @exception Exception
	     * @see #put( String, String )
	     * @see #putFile( String, String )
	     */

	    public long appendFile(String localFile, String remoteFile) throws Exception {
	    	return putFile(localFile,appendFileStream( remoteFile ));
	    } // appendFile



	    /**
	     * Using ASCII mode for file transfers
	     * @return True if successfully completed, false if not.
	     * @throws IOException If an I/O error occurs while either sending a
	     * command to the server or receiving a reply from the server.
	     */
	    public boolean ascii() throws IOException {
	    	return setFileType( FTP.ASCII_FILE_TYPE);
	    }
 
	    /**
	     * Using Binary mode for file transfers
	     * @return True if successfully completed, false if not.
	     * @throws IOException If an I/O error occurs while either sending a
	     * command to the server or receiving a reply from the server.
	     */
	    public boolean binary() throws IOException {
	    	return setFileType( FTP.BINARY_FILE_TYPE);
	    }

	    /**
	     *
	     * @param directory The new working directory.
	     * @return The reply code received from the server.
	     * @throws IOException If an I/O error occurs while either sending a
	     * command to the server or receiving a reply from the server.
	     */
	    public int cd(String directory) throws IOException {
	    	return cwd( directory );
	    }

	    /**
	     * Deletes a file on the FTP server.
	     * @param The pathname of the file to be deleted.
	     * @return True if successfully completed, false if not.
	     * @throws IOException If an I/O error occurs while either sending a
	     * command to the server or receiving a reply from the server.
	     */
	    public boolean delete(String pathname)
        throws IOException {
	    	return deleteFile(pathname);
	    }

}

