
package sos.settings;

import sos.util.SOSLogger;
import sos.util.SOSClassUtil;
import sos.util.SOSString;
import java.util.*;

/**
 * 
 * Diese Klasse interpretiert boolische Ausdrcke.
 * Die Boolischen Ausdrcke bestehen aus den Operanten = "and" und "or". Die Klein- und Groschreibung
 * der Operanten ist unwichtig.
 * 
 * 
 *  
 * 
 * Es existieren drei Konstruktoren. 
 * Konstruktor 1 und 2: zu den boolischen Ausdruck wird berprft, ob ein Wert in der Hashtabelle existiert. Wenn ja, dann
 * wird dieses feld als true interpretiert. Sonst als False.
 * Z.B. 	boolische Austdruck: (name and vorname and strasse and plz) or kundennummer
 * 	   		HashMap besteht aus = (name="otto", vorname="hans", plz ="", kundennummer="123")
 *         
 *          -> das entspricht: (true and true and false(*) and false) or true 
 *                  bzw.        (1 and 1 and 0 and 0) or 1 
 *         (*)da strasse nicht in der HashMap vorhanden ist, wird dieser auch als false interpretiert.
 * 
 * 1. Konstruktor besteht aus 3 Parametern:
 * - Property sektion mandantory als Property
 * - HashMap Werte zu den Pflichtfelder
 * - sosLogger Objekt
 * 
 * 2. Konstruktor besteht aus 3 Parametern:
 * - String Name der Konfigurationsdatei mit Verzeichnisangaben
 * - HashMap Werte zu den Pflichtfelder
 * - sosLogger Objekt
 * 
 * 3. Konstruktor besteht aus 2 Parametern:
 * - String condition in der Form (true and false) or false bzw. ((1 and 0) or 0)
 * - sosLogger Objekt
 * 
 * 
 * @author Mrvet ksz
 *
 */
public class SOSCheckSettings {
    
    /** Pfad und Name der Datenquelle ( Konfigurationstabelle) */
    private String source				= "";

   /** logger objekt */
    private SOSLogger logger			= null;
    
    /** Properties der Sektion mandantory */
    private Properties sectionMandatory = null;
    
    /** SOSString Object */
    private SOSString sosString    		= null;
    
    /** Attribut Ausgelesenen Text der Pflichtfelder aus der Sektion mandantory */
    private String condition 			= "";
    
    /** Attribut: Werte zu den Pflichtfelder. */
    private HashMap values				= null;
    
    /** Attribut: Liste der Regeln */
    private HashMap rules = new HashMap();
    
    
    /**
     * Konstruktor
     * @param String source_ -> Pfad und Name der Konfigurationsdatei zu der Sektion mandantory.
     * @param SOSLogger Objekt
     *   
     */
    public SOSCheckSettings(String source_, HashMap values_, SOSLogger logger_) throws Exception {
        try {
            source = source_;
            this.values = values_;
            logger =logger_;
            init();
            getSection();
            condition = sosString.parseToString(sectionMandatory,"mandatory"); 
            
        } catch (Exception e){
            throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
        }
                
    }

    /**
     * Konstruktor
     * @param Properties der Sektion Mandantory 
     * @param SOSLogger Objekt
     * @param HashMap values -> eventuell Vorhandene Daten zu der zu den Pflichtfelder.
     *   
     */
    public SOSCheckSettings(Properties sectionMandatory_, HashMap values_, SOSLogger logger_) throws Exception {
        try {
            this.sectionMandatory = sectionMandatory_;
            this.values = values_;
            logger = logger_;
            init();            
            condition = sosString.parseToString(sectionMandatory,"mandatory"); 
        } catch (Exception e){
            throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
        }       
    }
    
    /**
     * Konstruktor
     * @param String -> condition_ in der Form (true and/or false) bzw. (1 and/or 0)mit Klammern.
     */
    public SOSCheckSettings(String condition_, SOSLogger logger_) throws Exception {
        try {
            this.condition = condition_;      
            logger = logger_;
            values = new HashMap();
            values.put("true", "1");            
            values.put("1", "1");            
            init();            
        } catch (Exception e){
            throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
        }       
    }
    
    private void init() throws Exception {
        try {
            sosString = new SOSString();
            rules = new HashMap();
            
            //VerOderung
            rules.put("0or0", "0");
            rules.put("0or1", "1");
            rules.put("1or0", "1");
            rules.put("1or1", "1");
            rules.put("\\(0or0\\)", "0");
            rules.put("\\(0or1\\)", "1");
            rules.put("\\(1or0\\)", "1");
            rules.put("\\(1or1\\)", "1");
            
            
            //VerUNDung
            rules.put("0and0", "0");
            rules.put("0and1", "0");
            rules.put("1and0", "0");
            rules.put("1and1", "1");
            rules.put("\\(0and0\\)", "0");
            rules.put("\\(0and1\\)", "0");
            rules.put("\\(1and0\\)", "0");
            rules.put("\\(1and1\\)", "1");
            
            //XODER
            rules.put("0xor0", "0");
            rules.put("0xor1", "1");
            rules.put("1xor0", "1");
            rules.put("1xor1", "0");
            rules.put("\\(0xor0\\)", "0");
            rules.put("\\(0xor1\\)", "1");
            rules.put("\\(1xor0\\)", "1");
            rules.put("\\(1xor1\\)", "0");
            
            //allgemein 
            rules.put("\\(1\\)", "1");
            rules.put("\\(0\\)", "0");
            
        } catch (Exception e){
            throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
        }
    }
    
    /** 
     * berprft, ob das element in der condition existiert und einen
     * Wert besitzt.
     * @param element
     * @return
     * @throws Exception
     */
    public boolean check(String element) throws Exception {
        boolean retVal = false;
        try {
            if (sosString.parseToString(values, element).length() > 0) {
               return true; 
            } else {
               return false;
            }            
    	} catch (Exception e){
    	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
    	}
    }      
       
    
    
    /**
     * Auslesen der Bedingung, die in der Konfigurationsdatei stehen.
     * 
     * @return String
     * @throws Exception
     */
    public String getCondition() throws Exception {                            
        try {            
            return condition;
    	} catch (Exception e){
    	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
    	}

    }
    
    /**
     * Setzen der Werte der Condition. Die keys in der HashTable sind die Platzhalter 
     * in der condition. 
     * @param HashMap, dessen Keys Platzhalter fr die Condition ist.  
     * @throws Exception
     */
    public void setConditionValues(HashMap values_) throws Exception {
        try {
            values = values_; 
        } catch (Exception e){
    	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
    	}
    }
    
    
    
    /**
     * normalisiert den String
     * @param key
     * @return
     * @throws Exception
     */   
    private String normalizedString(String key) throws Exception {
        try {
            key = key.replaceAll("\\(", "");
            key = key.replaceAll("\\)", "");            
            key = key.replaceAll("  "," ");
            return key;
    	} catch (Exception e){
	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
	} 
    }
    
    /**
     * normalisiert den And und Or in der String hCond in kleinbuchstaben
     * @param key
     * @return String
     * @throws Exception
     */   
    private String normailzedANDOR(String hCond) throws Exception {
        try {
            hCond = hCond.replaceAll(" And ", " and ");
            hCond = hCond.replaceAll(" ANd ", " and ");
            hCond = hCond.replaceAll(" aNd ", " and ");
            hCond = hCond.replaceAll(" AND ", " and ");
            hCond = hCond.replaceAll(" AnD ", " and ");
            hCond = hCond.replaceAll(" aND ", " and ");            
            hCond = hCond.replaceAll(" anD ", " and ");            
            hCond = hCond.replaceAll(" OR ", " or ");
            hCond = hCond.replaceAll(" Or ", " or ");
            hCond = hCond.replaceAll(" oR ", " or ");
            return hCond;
    	} catch (Exception e){
	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
	} 
    }
    
    
    /**
     * Die Eintrge der Condition werden durch die Keys der conditionalValues ersetz.
     * 
     * Ist ein Key nicht vorhanden, dann ist es ein false.
     * Ist der Key leer bzw. null, dann ist der Wert false
     * @return
     * @throws Exception
     */
    public boolean process() throws Exception {
        boolean retVal = false;
        String hCond = ""; //hilfsvariable
        String[] cond = null; //hilfsvariable
        String result = "";
        try {
            checkCondition();
            checkValues();
            hCond = condition;          
            //hCond = normailzedANDOR(hCond);            
            cond = normalizedString(hCond).split(" and | or ");
            cond = this.sortDesc(cond, false);
            for (int i = 0; i < cond.length; i++) {
                if (sosString.parseToString(values,cond[i]).length() == 0) {
                    hCond = hCond.replaceAll(cond[i],"0"); 
                } else {
                    hCond = hCond.replaceAll(cond[i],"1");
                }
            }
            logger.info("..Pflichtfelder zum Auswerten ist: " + condition);
            logger.info("..entspricht : " + hCond);                                                
            hCond = hCond.trim();
            hCond = hCond.replaceAll(" ", "");
            
            while (hCond.length() != 1) {                
                Iterator keys = rules.keySet().iterator();
                Iterator vals = rules.values().iterator();            
                String key = "";
                String val = "";
                while (keys.hasNext() && hCond.length() != 1) {
                    key = keys.next().toString().trim();
                    val = vals.next().toString().trim();
                    logger.debug5(".. put rules [" + key + "=" + val + "] to condition = " + hCond);
                    hCond = hCond.replaceAll("  ", " ").replaceAll(key, val);
                    logger.debug5(".. result is = " + hCond);
                    
                }
            }
            if (hCond.equalsIgnoreCase("1")) {
                retVal = true;
            } else {
                retVal = false;
            }
            logger.info("..ergebnis ist : " + retVal);
            return retVal;
    	} catch (Exception e){
    	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
    	}

    }

    /**
     * Section mandantory auslesen auslesen
     * @throws Exception
     */
    public void getSection() throws Exception{        
        SOSProfileSettings settings = null;                
        try {                                               
           logger.debug4("reading settings ..." + source);                               
           settings	= new SOSProfileSettings(source, logger);            
           sectionMandatory = settings.getSection("mandatory"); 

        } catch (Exception e){
    	    throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
    	}
    }
    
    /**
     * 	Sort a String array using selection sort.
     *  desc boolean = true ist fr das Aufsteigende Sortierung
     *  desc boolean = false ist fr das Absteigende Sortierung
     * @param String[] eine Menge unsortierte String
     * @return String[] eine Menge der Sortierten String
     */  
	private String[] sortDesc(String[] a, boolean desc) throws Exception {
	    String[] aa = null; //Hilfsvariable, wenn absteigen Sortiert werden soll
	    int len = -1; //Hilfsvariable, lnge der Array
	    try {
	        for (int i=0; i < a.length-1; i++) {
	            for (int j=i+1; j<a.length; j++) {
	                if (a[i].compareTo(a[j]) > 0) {
	                    String temp=a[j]; 
	                    a[j]=a[i]; 
	                    a[i]=temp;
	                }
	            }
	        }
	        if (desc) {
	            return a;
	        } else {
	            len = a.length;
	            aa = new String[len];
	            for (int i = 0; i < len; i++) {
	                aa[len-1 - i] = a[i];
	            }
	            return aa;
	        }
	    } catch (Exception e){
	        throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
	    }
	}
	
	/**
     * berprft, ob der gegebene boolische Ausdruck ein gltiger Ausdruck ist.
     * 1. berprfung: Anzahl der geffneten Klammer == Anzahl der geschlossene Klammern
     * 2. Grosschreibung in Kleinschreibung umwandeln
     */
	private void checkCondition() throws Exception {
	    String hcon = null; // hilfsvariable
		try {
		    condition = condition.toLowerCase();	
		    hcon = condition;
			if (condition != null && condition.length() > 0) {
			    hcon = hcon.replaceAll("\\(", " \\( ");
			    hcon = hcon.replaceAll("\\)", " \\) ");			
				if (hcon.split("\\(").length != hcon.split("\\)").length) {			   
					throw new Exception("\n ..count of \"(\" is not equal of count of \")\" for condition : " + condition); 
				} 						
			}
		
		} catch (Exception e){
	        throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
	    }
		
	}
	
	
	/**
     * 
     * 2. Grosschreibung (key) in Kleinschreibung umwandeln
     */
	private void checkValues() throws Exception {
	    String hcon = null; // hilfsvariable
	    HashMap hValues = new HashMap();
		try {		    
	         Iterator keys = values.keySet().iterator();
	         Iterator vals = values.values().iterator();            
	         String key = "";
	         Object val = "";
	         while (keys.hasNext()) {
	             key = keys.next().toString().toLowerCase();
	             val = vals.next();
	             logger.debug5(".. " + key + "=" + val);
	             hValues.put(key, val);
	         }
	         if (values == null) {
	             values = new HashMap();
	         } else {
	             values.clear();
	         }
	         values.putAll(hValues);
		} catch (Exception e){
	        throw new Exception ("\n -> error in " + SOSClassUtil.getMethodName() + " " + e); 
	    }
		
	}
	
}