// $Id: xml.cxx 4915 2007-03-25 11:42:04Z jz $

#include "zschimmer.h"
#include "xml.h"
#include "z_io.h"


namespace zschimmer {
namespace xml {

//--------------------------------------------------------------------------------------encode_text
    
string encode_text( const string& value, bool quote )
{
    // Fr UTF-8 umstellen!

    string result;
    int reserve = value.length() + 10 + value.length() / 10;

    int last = -1;

    for( uint i = 0; i < value.length(); i++ )
    {
        switch( value[i] )
        {
            case '<': 
                if( result.length() == 0 )  result.reserve( reserve );
                result.append( value, last+1, i - (last+1) );
                result += "&lt;";
                last = i;
                break;

            case '>': 

                if( result.length() == 0  
                 || result.length() >= 1  &&  result[ result.length() - 1 ] == '-'  
                 || result.length() >= 2  &&  result[ result.length() - 2 ] == '-' )
                {
                    if( result.length() == 0 )  result.reserve( reserve );
                    result.append( value, last+1, i - (last+1) );
                    result.append( "&gt;" );
                    last = i;
                }
                break;

            case '&':
                if( result.length() == 0 )  result.reserve( reserve );
                result.append( value, last+1, i - (last+1) );
                result.append( "&amp;" );
                last = i;
                break;

            case '"':
                if( quote )
                {
                    if( result.length() == 0 )  result.reserve( reserve );
                    result.append( value, last+1, i - (last+1) );
                    result.append( "&quote;" );
                    last = i;
                }
                break;

            default: ;
        }
    }

    result.append( value, last+1, value.length() - (last+1) );

    return result;
}

//---------------------------------------------------------------------------Xml_writer::Xml_writer

Xml_writer::Xml_writer( Writer* writer )
: 
    Filter_writer(writer),
    _is_tag_open(false)
{

}

//----------------------------------------------------------------------------Xml_writer::close_tag
    
void Xml_writer::close_tag()
{
    if( _is_tag_open )
    {
        _writer->write( '>' );
        _is_tag_open = false;
    }
}

//------------------------------------------------------------------------Xml_writer::begin_element

void Xml_writer::begin_element( const string& element_name )
{
    if( _is_tag_open )  close_tag();

    _writer->write( '<' );
    _writer->write( element_name );
    _is_tag_open = true;
}

//------------------------------------------------------------------------Xml_writer::set_attribute

void Xml_writer::set_attribute( const string& name, int64 value )
{
    set_attribute( name, as_string( value ) );
}

//------------------------------------------------------------------------Xml_writer::set_attribute

void Xml_writer::set_attribute( const string& name, const string& value )
{
    _writer->write( ' ' );
    _writer->write( name );
    _writer->write( '=' );
    _writer->write( '"' );
    
    for( const char* p = value.c_str(); *p; p++ )
    {
        if( *p == '<' )  _writer->write( "&lt;" );
        else
        if( *p == '>' )  _writer->write( "&gt;" );
        else
        if( *p == '&' )  _writer->write( "&amp;" );
        else
        if( *p == '"' )  _writer->write( "&quot;" );
        else
            _writer->write( *p );
    }

    _writer->write( '"' );
}

//---------------------------------------------------------------Xml_writer::set_attribute_optional

void Xml_writer::set_attribute_optional( const string& name, const string& value )
{
    if( value != "" )  set_attribute( name, value );
}

//--------------------------------------------------------------------------Xml_writer::end_element

void Xml_writer::end_element( const string& element_name )
{
    if( _is_tag_open )  
    {
        _writer->write( '/' );
        _writer->write( '>' );
        _is_tag_open = false;
    }
    else
    {
        _writer->write( '<' );
        _writer->write( '/' );
        _writer->write( element_name );
        _writer->write( '>' );
    }
}

//--------------------------------------------------------------------------------Xml_writer::write

void Xml_writer::write( const io::Char_sequence& utf8_seq )
{
    if( utf8_seq.length() > 0 )
    {
        if( _is_tag_open )  close_tag();

        const char* p_end = utf8_seq.ptr() + utf8_seq.length();

        for( const char* p = utf8_seq.ptr(); p < p_end; p++ )
        {
            if( *p == '<' )  _writer->write( "&lt;" );
            else
            if( *p == '>' )  _writer->write( "&gt;" );
            else
            if( *p == '&' )  _writer->write( "&amp;" );
            else
            {
                const char* p0 = p;
                
                while( p < p_end  &&  *p != '<'  &&  *p != '&' )
                {
                    if( p[0] == '>' )
                    {
                        if( p < p0+1  ||  p[-1] == ']' )  break;    // ]>
                        if( p < p0+2  ||  p[-2] == ']' )  break;    // ]]>
                    }

                    p++;
                }

                _writer->write( io::Char_sequence( p0, p - p0 ) );
            }
        }
    }
}

//------------------------------------------------------------------------Xml_writer::write_through

void Xml_writer::write_through( const io::Char_sequence& seq )
{
    _writer->write( seq );
}

//--------------------------------------------------------------------------------Xml_writer::flush

void Xml_writer::flush()
{
}

//-------------------------------------------------------------------------------------------------

} //namespace xml
} //namespace zschimmer
